@@ -8,8 +8,8 @@ mod option;
 pub use self::{
     command_type::CommandType,
     option::{
-        BaseCommandOptionData, ChoiceCommandOptionData, CommandOption, CommandOptionChoice,
-        CommandOptionType, OptionsCommandOptionData,
+        BaseCommandOptionData, ChannelCommandOptionData, ChoiceCommandOptionData, CommandOption,
+        CommandOptionChoice, CommandOptionType, Number, OptionsCommandOptionData,
     },
 };
 
@@ -22,6 +22,10 @@ use serde::{Deserialize, Serialize};
 /// Command names must be lower case, matching the Regex `^[\w-]{1,32}$`. Refer
 /// to [the discord docs] for more information.
 ///
+/// This struct has an [associated builder] in the [`twilight-util`] crate.
+///
+/// [`twilight-util`]: https://docs.rs/twilight-util/latest/index.html
+/// [associated builder]: https://docs.rs/twilight-util/latest/builder/command/struct.CommandBuilder.html
 /// [the discord docs]: https://discord.com/developers/docs/interactions/application-commands#applicationcommand
 #[derive(Clone, Debug, Deserialize, Eq, Hash, PartialEq, Serialize)]
 pub struct Command {
