@@ -1,11 +1,15 @@
-use crate::util::is_false;
+use crate::{channel::ChannelType, util::is_false};
 use serde::{
     de::{Deserializer, Error as DeError, IgnoredAny, MapAccess, Visitor},
     ser::Serializer,
     Deserialize, Serialize,
 };
 use serde_repr::{Deserialize_repr, Serialize_repr};
-use std::fmt::{Formatter, Result as FmtResult};
+use std::{
+    cmp::Eq,
+    fmt::{Formatter, Result as FmtResult},
+    hash::{Hash, Hasher},
+};
 
 /// Option for a [`Command`].
 ///
@@ -25,9 +29,10 @@ pub enum CommandOption {
     Integer(ChoiceCommandOptionData),
     Boolean(BaseCommandOptionData),
     User(BaseCommandOptionData),
-    Channel(BaseCommandOptionData),
+    Channel(ChannelCommandOptionData),
     Role(BaseCommandOptionData),
     Mentionable(BaseCommandOptionData),
+    Number(ChoiceCommandOptionData),
 }
 
 impl CommandOption {
@@ -42,16 +47,19 @@ impl CommandOption {
             CommandOption::Channel(_) => CommandOptionType::Channel,
             CommandOption::Role(_) => CommandOptionType::Role,
             CommandOption::Mentionable(_) => CommandOptionType::Mentionable,
+            CommandOption::Number(_) => CommandOptionType::Number,
         }
     }
 
     pub const fn is_required(&self) -> bool {
         match self {
-            CommandOption::SubCommand(data) | CommandOption::SubCommandGroup(data) => data.required,
-            CommandOption::String(data) | CommandOption::Integer(data) => data.required,
+            CommandOption::SubCommand(_) | CommandOption::SubCommandGroup(_) => false,
+            CommandOption::String(data)
+            | CommandOption::Integer(data)
+            | CommandOption::Number(data) => data.required,
+            CommandOption::Channel(data) => data.required,
             CommandOption::Boolean(data)
             | CommandOption::User(data)
-            | CommandOption::Channel(data)
             | CommandOption::Role(data)
             | CommandOption::Mentionable(data) => data.required,
         }
@@ -66,6 +74,8 @@ impl<'de> Deserialize<'de> for CommandOption {
 
 #[derive(Serialize)]
 struct CommandOptionEnvelope<'ser> {
+    #[serde(skip_serializing_if = "Option::is_none")]
+    channel_types: Option<&'ser [ChannelType]>,
     #[serde(skip_serializing_if = "Option::is_none")]
     choices: Option<&'ser [CommandOptionChoice]>,
     description: &'ser str,
@@ -82,26 +92,27 @@ impl Serialize for CommandOption {
     fn serialize<S: Serializer>(&self, serializer: S) -> Result<S::Ok, S::Error> {
         let envelope = match self {
             Self::SubCommand(data) | Self::SubCommandGroup(data) => CommandOptionEnvelope {
+                channel_types: None,
                 choices: None,
                 description: data.description.as_ref(),
                 name: data.name.as_ref(),
                 options: Some(data.options.as_ref()),
-                required: data.required,
+                required: false,
                 kind: self.kind(),
             },
-            Self::String(data) | Self::Integer(data) => CommandOptionEnvelope {
-                choices: Some(data.choices.as_ref()),
-                description: data.description.as_ref(),
-                name: data.name.as_ref(),
-                options: None,
-                required: data.required,
-                kind: self.kind(),
-            },
-            Self::Boolean(data)
-            | Self::User(data)
-            | Self::Channel(data)
-            | Self::Role(data)
-            | Self::Mentionable(data) => CommandOptionEnvelope {
+            Self::String(data) | Self::Integer(data) | Self::Number(data) => {
+                CommandOptionEnvelope {
+                    channel_types: None,
+                    choices: Some(data.choices.as_ref()),
+                    description: data.description.as_ref(),
+                    name: data.name.as_ref(),
+                    options: None,
+                    required: data.required,
+                    kind: self.kind(),
+                }
+            }
+            Self::Channel(data) => CommandOptionEnvelope {
+                channel_types: Some(data.channel_types.as_ref()),
                 choices: None,
                 description: data.description.as_ref(),
                 name: data.name.as_ref(),
@@ -109,6 +120,17 @@ impl Serialize for CommandOption {
                 required: data.required,
                 kind: self.kind(),
             },
+            Self::Boolean(data) | Self::User(data) | Self::Role(data) | Self::Mentionable(data) => {
+                CommandOptionEnvelope {
+                    channel_types: None,
+                    choices: None,
+                    description: data.description.as_ref(),
+                    name: data.name.as_ref(),
+                    options: None,
+                    required: data.required,
+                    kind: self.kind(),
+                }
+            }
         };
 
         envelope.serialize(serializer)
@@ -118,6 +140,7 @@ impl Serialize for CommandOption {
 #[derive(Debug, Deserialize)]
 #[serde(field_identifier, rename_all = "snake_case")]
 enum OptionField {
+    ChannelTypes,
     Choices,
     Description,
     Name,
@@ -137,6 +160,7 @@ impl<'de> Visitor<'de> for OptionVisitor {
 
     #[allow(clippy::too_many_lines)]
     fn visit_map<V: MapAccess<'de>>(self, mut map: V) -> Result<Self::Value, V::Error> {
+        let mut channel_types: Option<Option<Vec<ChannelType>>> = None;
         let mut choices: Option<Option<Vec<CommandOptionChoice>>> = None;
         let mut description: Option<String> = None;
         let mut kind: Option<CommandOptionType> = None;
@@ -168,6 +192,13 @@ impl<'de> Visitor<'de> for OptionVisitor {
             };
 
             match key {
+                OptionField::ChannelTypes => {
+                    if choices.is_some() {
+                        return Err(DeError::duplicate_field("channel_types"));
+                    }
+
+                    channel_types = Some(map.next_value()?);
+                }
                 OptionField::Choices => {
                     if choices.is_some() {
                         return Err(DeError::duplicate_field("choices"));
@@ -234,7 +265,6 @@ impl<'de> Visitor<'de> for OptionVisitor {
                     description,
                     name,
                     options,
-                    required,
                 })
             }
             CommandOptionType::SubCommandGroup => {
@@ -244,7 +274,6 @@ impl<'de> Visitor<'de> for OptionVisitor {
                     description,
                     name,
                     options,
-                    required,
                 })
             }
             CommandOptionType::String => CommandOption::String(ChoiceCommandOptionData {
@@ -269,7 +298,8 @@ impl<'de> Visitor<'de> for OptionVisitor {
                 name,
                 required,
             }),
-            CommandOptionType::Channel => CommandOption::Channel(BaseCommandOptionData {
+            CommandOptionType::Channel => CommandOption::Channel(ChannelCommandOptionData {
+                channel_types: channel_types.flatten().unwrap_or_default(),
                 description,
                 name,
                 required,
@@ -284,6 +314,12 @@ impl<'de> Visitor<'de> for OptionVisitor {
                 name,
                 required,
             }),
+            CommandOptionType::Number => CommandOption::Number(ChoiceCommandOptionData {
+                choices: choices.flatten().unwrap_or_default(),
+                description,
+                name,
+                required,
+            }),
         })
     }
 }
@@ -325,9 +361,6 @@ pub struct OptionsCommandOptionData {
     /// [`SubCommandGroup`]: CommandOptionType::SubCommandGroup
     #[serde(default)]
     pub options: Vec<CommandOption>,
-    /// Whether the option is required to be completed by a user.
-    #[serde(default)]
-    pub required: bool,
 }
 
 /// Data supplied to a [`CommandOption`] of type [`String`] or [`Integer`].
@@ -353,6 +386,25 @@ pub struct ChoiceCommandOptionData {
     pub required: bool,
 }
 
+/// Data supplied to a [`CommandOption`] of type [`Channel`].
+///
+/// [`Channel`]: CommandOption::Channel
+#[derive(Clone, Debug, Default, Deserialize, Eq, Hash, PartialEq, Serialize)]
+pub struct ChannelCommandOptionData {
+    /// Restricts the channel choice to specific types.
+    ///
+    /// If no types are selected, all channel types can be sent.
+    #[serde(default)]
+    pub channel_types: Vec<ChannelType>,
+    /// Description of the option. It must be 100 characters or less.
+    pub description: String,
+    /// Name of the option. It must be 32 characters or less.
+    pub name: String,
+    /// Whether or not the option is required to be completed by a user.
+    #[serde(default)]
+    pub required: bool,
+}
+
 /// Specifies an option that a user must choose from in a dropdown.
 ///
 /// Refer to [the discord docs] for more information.
@@ -363,6 +415,7 @@ pub struct ChoiceCommandOptionData {
 pub enum CommandOptionChoice {
     String { name: String, value: String },
     Int { name: String, value: i64 },
+    Number { name: String, value: Number },
 }
 
 /// Type of a [`CommandOption`].
@@ -380,6 +433,7 @@ pub enum CommandOptionType {
     Channel = 7,
     Role = 8,
     Mentionable = 9,
+    Number = 10,
 }
 
 impl CommandOptionType {
@@ -394,19 +448,60 @@ impl CommandOptionType {
             CommandOptionType::Channel => "Channel",
             CommandOptionType::Role => "Role",
             CommandOptionType::Mentionable => "Mentionable",
+            CommandOptionType::Number => "Number",
         }
     }
 }
 
+#[derive(Clone, Copy, Debug, Deserialize, Serialize)]
+pub struct Number(pub f64);
+
+impl Eq for Number {}
+
+impl Hash for Number {
+    fn hash<H: Hasher>(&self, state: &mut H) {
+        self.0.to_bits().hash(state);
+    }
+}
+
+impl PartialEq for Number {
+    fn eq(&self, other: &Number) -> bool {
+        self.0.to_bits() == other.0.to_bits()
+    }
+}
+
+impl From<Number> for f64 {
+    fn from(number: Number) -> f64 {
+        number.0
+    }
+}
+
 #[cfg(test)]
 mod tests {
     use super::{
         super::{Command, CommandType},
-        BaseCommandOptionData, ChoiceCommandOptionData, CommandOption, CommandOptionChoice,
-        OptionsCommandOptionData,
+        BaseCommandOptionData, ChannelCommandOptionData, ChoiceCommandOptionData, CommandOption,
+        CommandOptionChoice, Number, OptionsCommandOptionData,
     };
-    use crate::id::{ApplicationId, CommandId, GuildId};
+    use crate::{
+        channel::ChannelType,
+        id::{ApplicationId, CommandId, GuildId},
+    };
+    use serde::{Deserialize, Serialize};
     use serde_test::Token;
+    use static_assertions::assert_impl_all;
+    use std::{fmt::Debug, hash::Hash};
+
+    assert_impl_all!(
+        Number: Clone,
+        Copy,
+        Debug,
+        Deserialize<'static>,
+        Eq,
+        Hash,
+        PartialEq,
+        Serialize
+    );
 
     /// Test that when a subcommand or subcommand group's `options` field is
     /// missing during deserialization that the field is defaulted instead of
@@ -417,7 +512,6 @@ mod tests {
             description: "ponyville".to_owned(),
             name: "equestria".to_owned(),
             options: Vec::new(),
-            required: false,
         });
 
         serde_test::assert_de_tokens(
@@ -444,13 +538,13 @@ mod tests {
     #[allow(clippy::too_many_lines)]
     fn test_command_option_full() {
         let value = Command {
-            application_id: Some(ApplicationId(100)),
-            guild_id: Some(GuildId(300)),
+            application_id: Some(ApplicationId::new(100).expect("non zero")),
+            guild_id: Some(GuildId::new(300).expect("non zero")),
             kind: CommandType::ChatInput,
             name: "test command".into(),
             default_permission: Some(true),
             description: "this command is a test".into(),
-            id: Some(CommandId(200)),
+            id: Some(CommandId::new(200).expect("non zero")),
             options: vec![CommandOption::SubCommandGroup(OptionsCommandOptionData {
                 description: "sub group desc".into(),
                 name: "sub group name".into(),
@@ -492,7 +586,8 @@ mod tests {
                             name: "user".into(),
                             required: false,
                         }),
-                        CommandOption::Channel(BaseCommandOptionData {
+                        CommandOption::Channel(ChannelCommandOptionData {
+                            channel_types: vec![ChannelType::GuildText],
                             description: "channel desc".into(),
                             name: "channel".into(),
                             required: false,
@@ -507,10 +602,17 @@ mod tests {
                             name: "mentionable".into(),
                             required: false,
                         }),
+                        CommandOption::Number(ChoiceCommandOptionData {
+                            choices: vec![CommandOptionChoice::Number {
+                                name: "choice3".into(),
+                                value: Number(2.0),
+                            }],
+                            description: "number desc".into(),
+                            name: "number".into(),
+                            required: false,
+                        }),
                     ],
-                    required: false,
                 })],
-                required: true,
             })],
         };
 
@@ -548,7 +650,7 @@ mod tests {
                 Token::Seq { len: Some(1) },
                 Token::Struct {
                     name: "CommandOptionEnvelope",
-                    len: 5,
+                    len: 4,
                 },
                 Token::Str("description"),
                 Token::Str("sub group desc"),
@@ -567,7 +669,7 @@ mod tests {
                 Token::Str("sub command name"),
                 Token::Str("options"),
                 Token::Some,
-                Token::Seq { len: Some(8) },
+                Token::Seq { len: Some(9) },
                 Token::Struct {
                     name: "CommandOptionEnvelope",
                     len: 4,
@@ -655,8 +757,13 @@ mod tests {
                 Token::StructEnd,
                 Token::Struct {
                     name: "CommandOptionEnvelope",
-                    len: 3,
+                    len: 4,
                 },
+                Token::Str("channel_types"),
+                Token::Some,
+                Token::Seq { len: Some(1) },
+                Token::U8(0),
+                Token::SeqEnd,
                 Token::Str("description"),
                 Token::Str("channel desc"),
                 Token::Str("name"),
@@ -686,13 +793,36 @@ mod tests {
                 Token::Str("type"),
                 Token::U8(9),
                 Token::StructEnd,
+                Token::Struct {
+                    name: "CommandOptionEnvelope",
+                    len: 4,
+                },
+                Token::Str("choices"),
+                Token::Some,
+                Token::Seq { len: Some(1) },
+                Token::Struct {
+                    name: "CommandOptionChoice",
+                    len: 2,
+                },
+                Token::Str("name"),
+                Token::Str("choice3"),
+                Token::Str("value"),
+                Token::NewtypeStruct { name: "Number" },
+                Token::F64(2.0),
+                Token::StructEnd,
+                Token::SeqEnd,
+                Token::Str("description"),
+                Token::Str("number desc"),
+                Token::Str("name"),
+                Token::Str("number"),
+                Token::Str("type"),
+                Token::U8(10),
+                Token::StructEnd,
                 Token::SeqEnd,
                 Token::Str("type"),
                 Token::U8(1),
                 Token::StructEnd,
                 Token::SeqEnd,
-                Token::Str("required"),
-                Token::Bool(true),
                 Token::Str("type"),
                 Token::U8(2),
                 Token::StructEnd,
@@ -701,4 +831,12 @@ mod tests {
             ],
         );
     }
+
+    #[test]
+    fn test_number() {
+        const NUMBER_1: Number = Number(12.34_f64);
+        const NUMBER_2: Number = Number(12.34_f64);
+
+        assert_eq!(NUMBER_1, NUMBER_2);
+    }
 }
