@@ -79,13 +79,14 @@ mod tests {
     use crate::{
         application::{component::ComponentType, interaction::InteractionType},
         channel::message::{Message, MessageType},
+        datetime::{Timestamp, TimestampParseError},
         guild::PartialMember,
         id::{ApplicationId, ChannelId, GuildId, InteractionId, MessageId, UserId},
         user::User,
     };
     use serde::Serialize;
     use static_assertions::{assert_fields, assert_impl_all};
-    use std::{fmt::Debug, hash::Hash};
+    use std::{fmt::Debug, hash::Hash, str::FromStr};
 
     assert_fields!(
         MessageComponentInteraction: application_id,
@@ -116,7 +117,7 @@ mod tests {
             avatar: None,
             banner: None,
             bot: false,
-            discriminator: "4444".to_owned(),
+            discriminator: 4444,
             email: None,
             flags: None,
             id,
@@ -131,19 +132,23 @@ mod tests {
     }
 
     #[test]
-    fn test_author_id() {
-        const USER_ID: UserId = UserId(7);
+    fn test_author_id() -> Result<(), TimestampParseError> {
+        fn user_id() -> UserId {
+            UserId::new(7).expect("non zero")
+        }
+
+        let timestamp = Timestamp::from_str("2020-02-02T02:02:02.020000+00:00")?;
 
         let in_guild = MessageComponentInteraction {
-            application_id: ApplicationId(1),
-            channel_id: ChannelId(2),
+            application_id: ApplicationId::new(1).expect("non zero"),
+            channel_id: ChannelId::new(2).expect("non zero"),
             data: MessageComponentInteractionData {
                 custom_id: "foo".to_owned(),
                 component_type: ComponentType::Button,
                 values: Vec::from(["bar".to_owned()]),
             },
-            guild_id: Some(GuildId(3)),
-            id: InteractionId(4),
+            guild_id: Some(GuildId::new(3).expect("non zero")),
+            id: InteractionId::new(4).expect("non zero"),
             kind: InteractionType::MessageComponent,
             member: Some(PartialMember {
                 deaf: false,
@@ -153,22 +158,22 @@ mod tests {
                 permissions: None,
                 premium_since: None,
                 roles: Vec::new(),
-                user: Some(user(USER_ID)),
+                user: Some(user(user_id())),
             }),
             message: Message {
                 activity: None,
                 application: None,
                 application_id: None,
                 attachments: Vec::new(),
-                author: user(USER_ID),
-                channel_id: ChannelId(5),
+                author: user(user_id()),
+                channel_id: ChannelId::new(5).expect("non zero"),
                 components: Vec::new(),
                 content: String::new(),
                 edited_timestamp: None,
                 embeds: Vec::new(),
                 flags: None,
-                guild_id: Some(GuildId(3)),
-                id: MessageId(6),
+                guild_id: Some(GuildId::new(3).expect("non zero")),
+                id: MessageId::new(6).expect("non zero"),
                 interaction: None,
                 kind: MessageType::Regular,
                 member: None,
@@ -181,7 +186,7 @@ mod tests {
                 reference: None,
                 referenced_message: None,
                 sticker_items: Vec::new(),
-                timestamp: String::new(),
+                timestamp,
                 thread: None,
                 tts: false,
                 webhook_id: None,
@@ -190,7 +195,7 @@ mod tests {
             user: None,
         };
 
-        assert_eq!(Some(USER_ID), in_guild.author_id());
+        assert_eq!(Some(user_id()), in_guild.author_id());
 
         let in_dm = MessageComponentInteraction {
             member: None,
@@ -198,9 +203,11 @@ mod tests {
                 guild_id: None,
                 ..in_guild.message
             },
-            user: Some(user(USER_ID)),
+            user: Some(user(user_id())),
             ..in_guild
         };
-        assert_eq!(Some(USER_ID), in_dm.author_id());
+        assert_eq!(Some(user_id()), in_dm.author_id());
+
+        Ok(())
     }
 }
