@@ -21,11 +21,15 @@
 //! resource category. These types may be directly returned by, built on top of,
 //! or extended by other crates.
 //!
+//! Some models have associated builders, which can be found in the
+//! [`twilight-util`] crate.
+//!
 //! ## License
 //!
 //! [ISC][LICENSE.md]
 //!
 //! [LICENSE.md]: https://github.com/twilight-rs/twilight/blob/main/LICENSE.md
+//! [`twilight-util`]: https://docs.rs/twilight-util
 //! [`twilight`]: https://docs.rs/twilight
 //! [discord badge]: https://img.shields.io/discord/745809834183753828?color=%237289DA&label=discord%20server&logo=discord&style=for-the-badge
 //! [discord link]: https://discord.gg/7jj8n7D
@@ -33,7 +37,7 @@
 //! [github link]: https://github.com/twilight-rs/twilight
 //! [license badge]: https://img.shields.io/badge/license-ISC-blue.svg?style=for-the-badge&logo=pastebin
 //! [license link]: https://github.com/twilight-rs/twilight/blob/main/LICENSE.md
-//! [rust badge]: https://img.shields.io/badge/rust-1.49+-93450a.svg?style=for-the-badge&logo=rust
+//! [rust badge]: https://img.shields.io/badge/rust-1.53+-93450a.svg?style=for-the-badge&logo=rust
 
 #![deny(
     clippy::all,
@@ -42,7 +46,7 @@
     future_incompatible,
     nonstandard_style,
     rust_2018_idioms,
-    broken_intra_doc_links,
+    rustdoc::broken_intra_doc_links,
     unsafe_code,
     unused,
     warnings
@@ -55,6 +59,7 @@
 
 pub mod application;
 pub mod channel;
+pub mod datetime;
 pub mod gateway;
 pub mod guild;
 pub mod id;
