@@ -1,8 +1,8 @@
-use super::{PremiumType, UserFlags};
+use super::{DiscriminatorDisplay, PremiumType, UserFlags};
 use crate::id::UserId;
 use serde::{Deserialize, Serialize};
 
-#[derive(Clone, Default, Debug, Deserialize, Eq, Hash, PartialEq, Serialize)]
+#[derive(Clone, Debug, Deserialize, Eq, Hash, PartialEq, Serialize)]
 pub struct CurrentUser {
     /// Accent color of the user's banner.
     ///
@@ -28,7 +28,7 @@ pub struct CurrentUser {
     /// integer. The field will always serialize into a string due to that being
     /// the type Discord's API uses.
     #[serde(with = "super::discriminator")]
-    pub discriminator: String,
+    pub discriminator: u16,
     /// User's email address associated to the account.
     ///
     /// Requires the `email` oauth scope. See [Discord's documentation] for
@@ -66,6 +66,15 @@ pub struct CurrentUser {
     pub verified: Option<bool>,
 }
 
+impl CurrentUser {
+    /// Create a [`Display`] formatter for a user discriminator.
+    ///
+    /// [`Display`]: core::fmt::Display
+    pub const fn discriminator(&self) -> DiscriminatorDisplay {
+        DiscriminatorDisplay::new(self.discriminator)
+    }
+}
+
 #[cfg(test)]
 mod tests {
     use super::{CurrentUser, PremiumType, UserFlags, UserId};
@@ -166,9 +175,9 @@ mod tests {
             avatar: Some("avatar hash".to_owned()),
             banner: None,
             bot: true,
-            discriminator: "9999".to_owned(),
+            discriminator: 9999,
             email: None,
-            id: UserId(1),
+            id: UserId::new(1).expect("non zero"),
             mfa_enabled: true,
             name: "test name".to_owned(),
             verified: Some(true),
@@ -195,9 +204,9 @@ mod tests {
             avatar: Some("avatar hash".to_owned()),
             banner: Some("06c16474723fe537c283b8efa61a30c8".to_owned()),
             bot: true,
-            discriminator: "9999".to_owned(),
+            discriminator: 9999,
             email: Some("test@example.com".to_owned()),
-            id: UserId(1),
+            id: UserId::new(1).expect("non zero"),
             mfa_enabled: true,
             name: "test name".to_owned(),
             verified: Some(true),
